$(document).ready(function () {
  app.initialized().then(function (_client) {
    client = _client;
    this.globalClient = client;
    client.data.get("domainName").then(
      function (data) {
        this.domainName = data.domainName;
      },
      function (error) {
        // failure operation
        console.log("unable to find domain name", error);
        this.domainName = "";
      }
    );
    client.events.on("app.activated", function (client) {
      getPowerbiToken(client).then(function () {
        getPowerbiReports(client);
      });
    });
  });
});

function createSideMenu(reports) {
  var sideMenu = document.getElementById("sideMenu");
  let _this = this;
  reports.forEach((report) => {
    var button = document.createElement("BUTTON");
    button.setAttribute("class", "menu-btn");
    button.setAttribute("data-item", JSON.stringify(report));
    button.innerHTML = report.report_name;
    button.onclick = function (evt) {
      evt.target.focus();
      const reportData = JSON.parse(evt.srcElement.getAttribute("data-item"));
      _this.createEmbedToken(reportData);
    };
    sideMenu.appendChild(button);
  });
  sideMenu.children[0].click();
}

function getPowerbiToken() {
  let tokenUrl =
    "https://middleware-prod.answeriq.com/api/_/middleware/powerbi/token";
  return this.globalClient.request
    .get(tokenUrl, {
      headers: {
        Authorization: "NmY2ZTlhNDgtOGYwZi00OTA5LTg0MzktMjliNDM5YzNhYjI2",
        "Content-Type": "application/json;charset=utf-8",
      },
    })
    .then(
      function (data) {
        let response = JSON.parse(data.response);
        this.savedToken = response;
      },
      function (error) {
        this.savedToken = { access_token: "" };
        document.getElementById("divDashBoardContainer").innerHTML = `
          <h4>Error in fetching power bi token from powerbi servers!</h4>`;
        console.log("Error in fetching powerbi token", error);
      }
    );
}

function getPowerbiReports() {
  let _this = this;
  let reportsUrl = `https://middleware-prod.answeriq.com/api/_/middleware/powerbi/reports?account_domain=${this.domainName}`;
  this.globalClient.request
    .get(reportsUrl, {
      headers: {
        Authorization: "NmY2ZTlhNDgtOGYwZi00OTA5LTg0MzktMjliNDM5YzNhYjI2",
        "Content-Type": "application/json;charset=utf-8",
      },
    })
    .then(
      function (data) {
        this.reports = JSON.parse(data.response);
        _this.createSideMenu(reports);
      },
      function (error) {
        document.getElementById("divDashBoardContainer").innerHTML = `
          <h4>${JSON.parse(error.response).msg}</h4>`;
        console.log("Error in fetching powerbi token", error);
      }
    );
}

function createEmbedToken(report) {
  let savedToken = this.savedToken.access_token;
  let accessToken = `Bearer ${savedToken}`;
  let type = "reports";
  let reportId = report.powerbi_url.split("=")[1].split("&")[0];
  let groupId = report.group_id || "ae5cb08b-7020-44aa-8c73-1b4c4966802e";
  let embedTokenUrl = `https://api.powerbi.com/v1.0/myorg/groups/${groupId}/${type}/${reportId}/GenerateToken`;
  client.request
    .post(embedTokenUrl, {
      headers: {
        Authorization: accessToken,
        "Content-Type": "application/json",
        "Cache-Control": "no-cache",
      },
      body: JSON.stringify({
        accessLevel: "View",
      }),
    })
    .then(function (res) {
      let response = JSON.parse(res.response);
      let embedTokenDetails = {
        embedToken: response.token,
        embedTokenExpiry: response.expiration,
      };
      return embedTokenDetails;
    })
    .then(
      function (token) {
        this.showReport(token, report.powerbi_url);
      },
      function (error) {
        console.log("Error in fetching powerbi embed token", error);
      }
    );
}

function showReport(token, url) {
  let models = window["powerbi-client"].models;
  let embedUrl = url.split("&autoAuth")[0];
  let id = url.split("=")[1].split("&")[0];
  let permissions = models.Permissions.All;
  let config = {
    type: "report",
    tokenType: models.TokenType.Embed,
    accessToken: token.embedToken,
    embedUrl,
    id,
    permissions,
    settings: {
      filterPaneEnabled: true,
      navContentPaneEnabled: true,
    },
  };
  let dashboardContainer = $("#divDashBoardContainer")[0];
  powerbi.embed(dashboardContainer, config);
}
